from OFS import ZDOM

class XMLStandardWidgetFinder:
    """Standard XML Widget finder.
    """
    def __init__(self):
	self.init()

    def init(self):
	"""Explicit initialization.
	"""
	# the map used for widgets in parent context
	self._context_map = {}
	# the map used for widgets in any context 
	# (parent context did not find any widget)
	self._default_map = {}
	# non persistent map for selectively rendering individual 
	# nodes, selected by id.
	self._v_special_map = {}
	
    def setWidget(self, widget, node_info, parent_info=None):
	"""Add widget to translation map.
	
	widget      - the widget
	node_info   - tuple of node type, node name
	parent_info - tuple of parent node type, node name 
	              (or None if we don't care about parent)
        """
	if parent_info is not None:
	    context_map = self._context_map
	    if not context_map.has_key(parent_info):
		context_map[parent_info] = { node_info : widget }
	    else:
		context_map[parent_info][node_info] = widget
	    self._context_map = context_map
	else:
	    default_map = self._default_map
	    default_map[node_info] = widget
	    self._default_map = default_map

    def removeWidget(self, node_info, parent_info=None):
	"""Remove a widget from the translation map.
	
	node_info   - tuple of node type, node name
	parent_info - tuple of parent node type, node name
	              (or None if no parent)
	"""
	if parent_info is not None:
	    del self._context_map[parent_info][node_info]
	else:
	    del self._default_map[node_info]
    
    def clearWidgets(self):
	"""Clear translation map.
	"""
	self._context_map = {}
	self._default_map = {}
	
    def setSpecialWidget(self, widget, node):
	"""Special rendering for node.

	widget - widget to use for special rendering
	node     - XMLNode for which to do special rendering
	"""
	self._v_special_map[node.absolute_url()] = widget
    
    def removeSpecialWidget(self, node):
	"""Remove special rendering for node.

	node - XMLNode to remove from special map
	"""
	del self._v_special_map[node.absolute_url()]
    
    def clearSpecialWidgets(self):
	"""Remove all special translations.
	"""
	self._v_special_map = {}

    def getSpecialWidgets(self):
	"""Get special widget info for debugging purposes.
	"""
	result = ""
	for key in self._v_special_map.keys():
	    result = result + ("[%s]" % key)
	return result

    def getWidget(self, node):
	"""Get the widget to use for a certain node (in its context).
	
	node   - the node to find the widget for
	result - the widget found, or None if no widget is found 
	"""
	# if we have a special map defined
	if hasattr(self, '_v_special_map'):
	    # if node id in special map, use that widget
	    widget = self._v_special_map.get(node.absolute_url(), None)
	    if widget is not None:
		return widget

	# get info for parent node
	parent = node.getParentNode()
	if parent is not None:
	    parent_info = get_node_info(parent)
	else:
	    parent_info = None # no parent

	# find translation map for parent context now (may be empty)
	trans_map = self._context_map.get(parent_info, {})

	# get info for this node
	node_info = get_node_info(node)
	
	# now use map to find widget -- if can't be found in context look
	# in default map
	widget = trans_map.get(node_info, None) or\
		 self._default_map.get(node_info, None)

	# return the widget
	return widget

def get_node_info(node):
    """Extract node info from node.

    node   - the node to extract node info from
    result - tuple of nodetype, nodename in case of element nodes, otherwise
             nodetype, None
    """
    node_type = node.getNodeType()
    if node_type == ZDOM.ELEMENT_NODE:
	return (node_type, node.getNodeName())
    else:
	return (node_type, None)
