from XMLWidget import XMLWidgetBase
from cStringIO import StringIO
import XMLSupport
from OFS import ZDOM
from DateTime import DateTime
import string

class Error(Exception):
    """Simple exception class for this module."""
    pass

def configure(manager):
    """Configure manager to deal with (subset of) ZAML
    """
    # get widgetfinder (that keeps map from XML node -> XML Widget)
    wf = manager.widgetfinder
    # clear widget finder from previous contents
    wf.clearWidgets()
    wf.clearSpecialWidgets()

    # define structure for rendering

    # 'zaml' node is rendered by ZamlWidget, can be contained by anything.
    wf.setWidget(ZamlWidget(), 
		 (ZDOM.ELEMENT_NODE, 'zaml'),
		 None)
    # a 'section' node contained in a 'zaml' node is rendered by SectionWidget
    wf.setWidget(SectionWidget(),
		 (ZDOM.ELEMENT_NODE, 'section'),
		 (ZDOM.ELEMENT_NODE, 'zaml'))
    # a 'p' node contained in a 'section' node is rendered by PWidget
    wf.setWidget(PWidget(),
		 (ZDOM.ELEMENT_NODE, 'p'),
		 (ZDOM.ELEMENT_NODE, 'section'))
    # an 'image' node contained in a 'section' node is rendered by ImageWidget
    wf.setWidget(ImageWidget(),
		 (ZDOM.ELEMENT_NODE, 'image'),
		 (ZDOM.ELEMENT_NODE, 'section'))

class ZamlWidget(XMLWidgetBase):
    """Widget for 'zaml' node.
    """
    def render(self, manager, node, REQUEST):
	result = StringIO()
	# render title
	result.write("<h2>%s</h2>\n" % node.title)
	# render children
	result.write(render_children(manager, node, REQUEST))
	return result.getvalue()

class SectionWidget(XMLWidgetBase):
    """Widget for 'section' node.
    """
    def __init__(self):
	XMLWidgetBase.__init__(self)
	self.allow_event('focusEvent')
	
    def render(self, manager, node, REQUEST):
	# register event
	event_id = manager.eventrouter.register_event('focusEvent', self, node)

	result = StringIO()
	# render title
	result.write('''
	<h3><a href="%s"><img src="test_gif" border="0"></a> %s</h3>
	''' % (event_url('zaml_view', event_id), node.title))
	# render children
	result.write(render_children(manager, node, REQUEST))
	return result.getvalue()
    
    def focusEvent(self, manager, node, REQUEST):
	# render the focused section differently
	manager.widgetfinder.setSpecialWidget(SectionSelectedWidget(), node)

class SectionSelectedWidget(XMLWidgetBase):
    """Widget for selected 'section' node.
    """
    def __init__(self):
	XMLWidgetBase.__init__(self)
	self.allow_event('editEvent')
	self.allow_event('aboveEvent')
	self.allow_event('topEvent')
	self.allow_event('belowEvent')
	self.allow_event('bottomEvent')
	self.allow_event('deleteEvent')

    def render(self, manager, node, REQUEST):
	# register events with eventrouter
	register_event = manager.eventrouter.register_event	
	edit_event_id = register_event('editEvent', self, node)
	above_event_id = register_event('aboveEvent', self, node)
	top_event_id = register_event('topEvent', self, node)
	below_event_id = register_event('belowEvent', self, node)
	bottom_event_id = register_event('bottomEvent', self, node)
	delete_event_id = register_event('deleteEvent', self, node)
	
	# create hyperlinks
	edit_url = event_url('zaml_view', edit_event_id)
	above_url = event_url('zaml_view', above_event_id)
	top_url = event_url('zaml_view', top_event_id)
	below_url = event_url('zaml_view', below_event_id)
	bottom_url = event_url('zaml_view', bottom_event_id)
	delete_url = event_url('zaml_view', delete_event_id)

	result = StringIO()
	# table to make background grey
	result.write('<a name="focus">')
	
	result.write('<a href="%s">Above</a>' % above_url)
	result.write(' | <a href="%s">Top</a>' % top_url)
	result.write(' | <a href="%s">Edit</a>' % edit_url)
	result.write(' | <a href="%s">Delete</a>' % delete_url)

	result.write('<table bgcolor="#777777"><tr><td>')
	# render title
	result.write("<h3>%s</h3>\n" % node.title)
	# render children
	result.write(render_children(manager, node, REQUEST))
	# end table
	result.write("</td></tr></table>")
	result.write('<a href="%s">Below</a>' % below_url)
	result.write(' | <a href="%s">Bottom</a>' % bottom_url)
	result.write("</a>")
	return result.getvalue()
    
    def editEvent(self, manager, node, REQUEST):
	manager.widgetfinder.setSpecialWidget(SectionEditableWidget(), node)
	
    def aboveEvent(self, manager, node, REQUEST):
	# insert new section node above this one
	section = XMLSupport.insertAbove(node, "section")
	# set title of new section
	section.title = "New Section"
	# edit new section
	manager.widgetfinder.setSpecialWidget(SectionEditableWidget(),
					      section)

    def belowEvent(self, manager, node, REQUEST):
	# insert new section node below this one
	section = XMLSupport.insertBelow(node, "section")
	# set title of new section
	section.title = "New Section"
	# edit new section
	manager.widgetfinder.setSpecialWidget(SectionEditableWidget(),
					      section)

    def topEvent(self, manager, node, REQUEST):
	# insert new paragraph at top of this node
	p = XMLSupport.insertTop(node, "p")
	# set text of new paragraph
	XMLSupport.setText(p, "New Paragraph")
	# edit new paragraph
	manager.widgetfinder.setSpecialWidget(PEditableWidget(), 
					      p)

    def bottomEvent(self, manager, node, REQUEST):
	# insert new paragraph at bottom fo this node
	p = XMLSupport.insertBottom(node, "p")
	# set text of new paragraph
	XMLSupport.setText(p, "New Paragraph")
	# edit new paragraph
	manager.widgetfinder.setSpecialWidget(PEditableWidget(),
					      p)

    def deleteEvent(self, manager, node, REQUEST):
	# find element node to focus, either below this one or if none found,
	# above it
	focusNode = (XMLSupport.findBelow(node) or
		     XMLSupport.findAbove(node))
	# remove node
	node.getParentNode().removeChild(node)
	# focus focusNode if there's something to focus
	if focusNode is not None:
	    manager.widgetfinder.setSpecialWidget(SectionSelectedWidget(),
						  focusNode)
	else:
	    # otherwise focus zaml node?
	    pass

class SectionEditableWidget(XMLWidgetBase):
    """Widget for editable 'section' node 
    """
    def __init__(self):
	XMLWidgetBase.__init__(self)
	self.allow_event('editDoneEvent')

    def render(self, manager, node, REQUEST):
	# register event
	event_id = manager.eventrouter.register_event('editDoneEvent', 
						      self, node)
	
	# create semi random id to make sure no duplicate request 
	# (not sure if this is necessary for forms but it doesn't hurt)
	
	hack = str(int(DateTime()))
	
	result = StringIO()
	result.write('''
	<a name="focus">
	<form action="zaml_done">
	<table border="0">
	  <tr><td>
	  <textarea name="field_text" cols="60" rows="2">%s</textarea>
	  </td></tr>
	  <tr><td>
  	  <input type="submit" value="Submit">
	  </td></tr>
        </table>
	<input type="hidden" name="event_id" value="%s">
	<input type="hidden" name="hack" value="%s">
	</form>
	</a>
	''' % (node.title, event_id, hack))

	result.write(render_children(manager, node, REQUEST))

	return result.getvalue()

    def editDoneEvent(self, manager, node, REQUEST):
	# set new title
	node.title = REQUEST.field_text
	# refocus edited section
	manager.widgetfinder.setSpecialWidget(SectionSelectedWidget(), node)

class PWidget(XMLWidgetBase):
    """Widget for 'p' node.
    """
    def __init__(self):
	XMLWidgetBase.__init__(self)
	self.allow_event('focusEvent')

    def render(self, manager, node, REQUEST):
	# register event
	event_id = manager.eventrouter.register_event('focusEvent', self, node)
    	# return html
	return '''
	<p><a href="%s"><img src="test_gif" border="0"></a> %s</p>
	''' % (event_url("zaml_view", event_id), 
	       render_text(node))
    
    def focusEvent(self, manager, node, REQUEST):
	# render the focused paragraph differently
	manager.widgetfinder.setSpecialWidget(PSelectedWidget(), node)

class PSelectedWidget(XMLWidgetBase):
    """Widget for selected 'p' node.
    """
    def __init__(self):
	XMLWidgetBase.__init__(self)
	# allow events
	self.allow_event('editEvent')
	self.allow_event('aboveEvent')
	self.allow_event('belowEvent')
	self.allow_event('deleteEvent')

    def render(self, manager, node, REQUEST):
	# register events with eventrouter
	register_event = manager.eventrouter.register_event	
	edit_event_id = register_event('editEvent', self, node)
	above_event_id = register_event('aboveEvent', self, node)
	below_event_id = register_event('belowEvent', self, node)
	delete_event_id = register_event('deleteEvent', self, node)
	
	# create hyperlinks
	edit_url = event_url('zaml_view', edit_event_id)
	above_url = event_url('zaml_view', above_event_id)
	below_url = event_url('zaml_view', below_event_id)
	delete_url = event_url('zaml_view', delete_event_id)
	
	return '''
	<a name="focus">
	<a href="%s">Above</a> | <a href="%s">Edit</a> | 
	<a href="%s">Delete</a>
	<table bgcolor="#777777"><tr><td>%s</td></tr></table>
	<a href="%s">Below</a>
	</a>
	''' % (above_url, edit_url, delete_url, render_text(node), below_url)
    
    def editEvent(self, manager, node, REQUEST):
	manager.widgetfinder.setSpecialWidget(PEditableWidget(), node)
    
    def aboveEvent(self, manager, node, REQUEST):
	# insert new p node above this one
	p = XMLSupport.insertAbove(node, "p")
	# set text
	XMLSupport.setText(p, "New Paragraph")

	# render p node editable
	manager.widgetfinder.setSpecialWidget(PEditableWidget(),
					      p)

    def belowEvent(self, manager, node, REQUEST):
	# insert new p node below this one
	p = XMLSupport.insertBelow(node, "p")
	# set text
	XMLSupport.setText(p, "New Paragraph")
	# render p node editable
	# HACK: need to get at it with nextSibling to make it work
	manager.widgetfinder.setSpecialWidget(PEditableWidget(), 
					      p)

    def deleteEvent(self, manager, node, REQUEST):
	# find element node to focus, either below this one or if none found,
	# above it
	focusNode = (XMLSupport.findBelow(node) or
		     XMLSupport.findAbove(node))
	# remove node
	node.getParentNode().removeChild(node)
	# focus focusNode if there's something to focus
	if focusNode is not None:
	    manager.widgetfinder.setSpecialWidget(PSelectedWidget(),
						  focusNode)
	else:
	    # otherwise focus section node?
	    pass

class PEditableWidget(XMLWidgetBase):
    """Widget for editable 'p' node.
    """
    def __init__(self):
	XMLWidgetBase.__init__(self)
	self.allow_event('editDoneEvent')

    def render(self, manager, node, REQUEST):
	# register event
	event_id = manager.eventrouter.register_event('editDoneEvent', 
						      self, node)
	
	# create semi random id to make sure no duplicate request 
	# (not sure if this is necessary for forms but it doesn't hurt)
	
	hack = str(int(DateTime()))
	
	return '''
	<a name="focus">
	<form action="zaml_done">
	<table border="0">
	  <tr><td>
	  <textarea name="field_text" cols="60" rows="5">%s</textarea>
	  </td></tr>
	  <tr><td>
  	  <input type="submit" value="Submit">
	  </td></tr>
        </table>
	<input type="hidden" name="event_id" value="%s">
	<input type="hidden" name="hack" value="%s">
	</form>
	</a>
	''' % (render_text_editable(node), event_id, hack)
    
    def editDoneEvent(self, manager, node, REQUEST):
	# set new text
	XMLSupport.setText(node, REQUEST.field_text)
	# refocus edited paragraph
	manager.widgetfinder.setSpecialWidget(PSelectedWidget(), node)
	
class ImageWidget(XMLWidgetBase):
    """Widget for 'image' node.
    """
    def render(self, manager, node, REQUEST):
	return '<img src="%s" alt="%s" border="">' % (
	    node.url, node.caption)

#####################
# Utility functions #
#####################

def render_children(manager, node, REQUEST):
    """Renders element child nodes.
    """
    result = StringIO()
    children = XMLSupport.only_element_nodes(node.getChildNodes())
    for child in children:
	result.write(manager.render(child, REQUEST))
    return result.getvalue()

def event_url(page, event_id):
    """Construct an url to page with event_id.
    """
    # construct url that handles event
    # need to add random hack url to make the page reload on same url
    return "%s?event_id=%s&hack=%s#focus" % ("zaml_view",
					     event_id, str(int(DateTime())))

def render_text(textcontainer):
    """Textcontainer is a node that contains textnodes and
    special nodes (strong, em and link nodes). HTML is generated.
    """
    result = StringIO()
    # content may be text node or strong/em/link
    for node in textcontainer.getChildNodes():
	# if we are dealing with an element node
	if node.getNodeType() == ZDOM.ELEMENT_NODE:
	    name = node.getNodeName()
	    # <strong>
	    if name == "strong":
		result.write("<strong>%s</strong>" % node.text_content())
	    elif name == "em":
		result.write("<em>%s</em>" % node.text_content())
	    elif name == "link":
		result.write("<a href=\"%s\">%s</a>" %
			     (node.url, node.text_content()))
	    else:
		raise Error,\
		      "'%s' found, but 'strong',"\
		      "'em' or 'link' expected." % name
	# add any text nodes directly
        elif node.getNodeType() == ZDOM.TEXT_NODE:
	    result.write(node.getNodeValue())
	# skip comments
        elif node.getNodeType() == ZDOM.COMMENT_NODE:
	    continue
	# if we can't handle this node, give error
	else:
	    raise Error, "Unexpected node type in render_text."
    return result.getvalue()

def render_text_editable(textcontainer, cols=60):
    """Textcontainer is a node that contains textnodes and
    special nodes (strong, em and link nodes). Text appropriate for
    a textarea is generated.
    """
    result = StringIO()
    # content may be text node or strong/em/link
    for node in textcontainer.getChildNodes():
	# if we are dealing with an element node
	if node.getNodeType() == ZDOM.ELEMENT_NODE:
	    name = node.getNodeName()
	    # <strong>
	    if name == "strong":
		result.write("_%s_" % node.text_content())
	    elif name == "em":
		result.write("*%s*" % node.text_content())
	    elif name == "link":
		result.write("__%s(%s)__" %
			     (node.text_content(), node.url))
	    else:
		raise Error,\
		      "'%s' found, but 'strong',"\
		      "'em' or 'link' expected." % name
	# add any text nodes directly
        elif node.getNodeType() == ZDOM.TEXT_NODE:
	    result.write(node.getNodeValue())
	# skip comments
        elif node.getNodeType() == ZDOM.COMMENT_NODE:
	    continue
	# if we can't handle this node, give error
	else:
	    raise Error, "Unexpected node type in render_text."
    # redo linebreaks
    lines = reline(result.getvalue(), cols)
    # now make long string again and return it
    return string.join(lines, "\n")

def reline(data, cols):
    """Put linebreaks at word boundaries.

    data - string with text to put linebreaks in
    cols - maximum amount of characters a single line can contain

    NOTE: any previous whitespace in data except single spaces between words
    is removed.
    """
    # make words
    words = string.split(data)
    # the lines that will be the result
    lines = []
    # current line
    line = ""

    i = 0
    while i < len(words):
        if len(line) + len(words[i]) + 1 < cols:
            # next word still fits on line, add it
            line = "%s%s " % (line, words[i])
        else:
            # okay, we're done with this line, add it
            lines.append(line)
            if len(words[i]) + 1 >= cols:
                # next word doesn't fit on a line anyway, 
		# so put it on a sep. line
                lines.append(words[i])
                line = ""
            else:
                # word does fit on next line
                line = words[i] + " "
        i = i + 1
    lines.append(line)
    return lines
