from XMLWidget import XMLWidgetBase
from cStringIO import StringIO
import XMLSupport
from OFS import ZDOM
from DateTime import DateTime
import string

class Error(Exception):
    """Simple exception class for this module."""
    pass

def configure(manager):
    """Configure manager to deal with (subset of) ZAML
    """
    # get widgetfinder (that keeps map from XML node -> XML Widget)
    wf = manager.widgetfinder
    # clear widget finder from previous contents
    wf.clearWidgets()
    wf.clearSpecialWidgets()

    # define structure for rendering

    # 'zaml' node is rendered by ZamlWidget, can be contained by anything.
    wf.setWidget(ZamlWidget(), 
		 (ZDOM.ELEMENT_NODE, 'zaml'),
		 None)
    # a 'section' node contained in a 'zaml' node is rendered by SectionWidget
    wf.setWidget(SectionWidget(),
		 (ZDOM.ELEMENT_NODE, 'section'),
		 (ZDOM.ELEMENT_NODE, 'zaml'))
    # a 'p' node contained in a 'section' node is rendered by PWidget
    wf.setWidget(PWidget(),
		 (ZDOM.ELEMENT_NODE, 'p'),
		 (ZDOM.ELEMENT_NODE, 'section'))
    # an 'image' node contained in a 'section' node is rendered by ImageWidget
    wf.setWidget(ImageWidget(),
		 (ZDOM.ELEMENT_NODE, 'image'),
		 (ZDOM.ELEMENT_NODE, 'section'))
    # a 'list' node contained in a 'section' node is rendered by ListWidget
    wf.setWidget(ListWidget(),
		 (ZDOM.ELEMENT_NODE, 'list'),
		 (ZDOM.ELEMENT_NODE, 'section'))
    # a 'li' node contained in a 'list' node is rendered by LiWidget
    wf.setWidget(LiWidget(),
		 (ZDOM.ELEMENT_NODE, 'li'),
		 (ZDOM.ELEMENT_NODE, 'list'))
    # a 'table' node contained in a 'section' node
    wf.setWidget(TableWidget(),
		 (ZDOM.ELEMENT_NODE, 'table'),
		 (ZDOM.ELEMENT_NODE, 'section'))
    wf.setWidget(RecordWidget(),
		 (ZDOM.ELEMENT_NODE, 'record'),
		 (ZDOM.ELEMENT_NODE, 'table'))
    wf.setWidget(FieldWidget(),
		 (ZDOM.ELEMENT_NODE, 'field'),
		 (ZDOM.ELEMENT_NODE, 'record'))

class ZamlWidget(XMLWidgetBase):
    """Widget for 'zaml' node.
    """
    def render(self, manager, node, REQUEST):
	result = StringIO()
	# render title
	result.write("<h2>%s</h2>\n" % node.title)
	# render children
	result.write(render_children(manager, node, REQUEST))
	return result.getvalue()

class SectionWidget(XMLWidgetBase):
    """Widget for 'section' node.
    """
 
    def render(self, manager, node, REQUEST):
	result = StringIO()
	# render title
	result.write('''
	<h3>%s</h3>
	''' % node.title)
	# render children
	result.write(render_children(manager, node, REQUEST))
	return result.getvalue()

class PWidget(XMLWidgetBase):
    """Widget for 'p' node.
    """
    def render(self, manager, node, REQUEST):
    	# return html
	return '''
	<p>%s</p>
	''' % render_text(node)
    	
class ImageWidget(XMLWidgetBase):
    """Widget for 'image' node.
    """
    def render(self, manager, node, REQUEST):
	return '<img src="%s" alt="%s" border="">' % (
	    node.url, node.caption)

class ListWidget(XMLWidgetBase):
    """Widget for 'list' node.
    """
    def render(self, manager, node, REQUEST):
	ordered = 0
	if hasattr(node, 'ordered') and node.ordered == 'yes':
	    ordered = 1
	
	result = StringIO()
	# begin of list
	if ordered:
	    result.write('<ol>')
	else:
	    result.write('<ul>')
	# render children
	result.write(render_children(manager, node, REQUEST))
	# end of list
	if ordered:
	    result.write('</ol>')
	else:
	    result.write('</ul>')
	return result.getvalue()

class LiWidget(XMLWidgetBase):
    def render(self, manager, node, REQUEST):
	return '''
	<li>%s</li>
	''' % render_text(node)

class TableWidget(XMLWidgetBase):
    def render(self, manager, node, REQUEST):
	result = StringIO()
	result.write('<table border="1">\n')
	result.write(render_children(manager, node, REQUEST))
	result.write('</table>\n')
	return result.getvalue()

class RecordWidget(XMLWidgetBase):
    def render(self, manager, node, REQUEST):
	result = StringIO()
	result.write('<tr>\n')
	result.write(render_children(manager, node, REQUEST))
	result.write('</tr>\n')
	return result.getvalue()

class FieldWidget(XMLWidgetBase):
    def render(self, manager, node, REQUEST):
	result = StringIO()
	result.write('<td>\n')
	result.write(render_text(node))
	result.write('</td>\n')
	return result.getvalue()

#####################
# Utility functions #
#####################

def render_children(manager, node, REQUEST):
    """Renders element child nodes.
    """
    result = StringIO()
    children = XMLSupport.only_element_nodes(node.getChildNodes())
    for child in children:
	result.write(manager.render(child, REQUEST))
    return result.getvalue()

def render_text(textcontainer):
    """Textcontainer is a node that contains textnodes and
    special nodes (strong, em and link nodes). HTML is generated.
    """
    result = StringIO()
    # content may be text node or strong/em/link
    for node in textcontainer.getChildNodes():
	# if we are dealing with an element node
	if node.getNodeType() == ZDOM.ELEMENT_NODE:
	    name = node.getNodeName()
	    # <strong>
	    if name == "strong":
		result.write("<strong>%s</strong>" % node.text_content())
	    elif name == "em":
		result.write("<em>%s</em>" % node.text_content())
	    elif name == "link":
		result.write("<a href=\"%s\">%s</a>" %
			     (node.url, node.text_content()))
	    else:
		raise Error,\
		      "'%s' found, but 'strong',"\
		      "'em' or 'link' expected." % name
	# add any text nodes directly
        elif node.getNodeType() == ZDOM.TEXT_NODE:
	    result.write(node.getNodeValue())
	# skip comments
        elif node.getNodeType() == ZDOM.COMMENT_NODE:
	    continue
	# if we can't handle this node, give error
	else:
	    raise Error, "Unexpected node type in render_text."
    return result.getvalue()
